// Slickworm
// Copyright (c) 2002-2004, Michel O'Brien.
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  1. Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//  2. Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//  3. Neither the names of the copyright holders nor the names of its
//     contributors may be used to endorse or promote products derived from this
//     software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
// IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#ifndef AUDIOSYSTEMOPENAL_H
#define AUDIOSYSTEMOPENAL_H

#if defined(WIN32)
#   define USE_SOUND 1
#endif // defined(WIN32)
#if defined(HAVE_CONFIG_H)
#   include "config.h"
#   if defined(HAVE_SOUND)
#       define USE_SOUND 1
#   endif // defined(HAVE_SOUND)
#endif // defined(HAVE_CONFIG_H)
#if defined(USE_SOUND)

#include <AL/al.h>
#include <AL/alc.h>
#include <string>
#include "mtxlib.h"

class OpenAL
{
public:
    OpenAL();
    ~OpenAL();

    // Initialises OpenAL. Returns 0 if initialisation was successful.
    int Init();

    // Shutsdown OpenAL.
    int Shutdown();

    void SetListenerPosition(vector3 pos);
    void SetListenerVelocity(vector3 vel);
    void SetListenerOrientation(vector3 lookat, vector3 up);

    // Creates an OpenAL Buffer.
    ALuint CreateBuffer();

    // Deletes a Buffer.
    void DeleteBuffer(ALuint buffer) { alDeleteBuffers(1, &buffer); }

    // Creates an OpenAL Source
    ALuint CreateSource();
    
    // Deletes a Source.
    void DeleteSource(ALuint source) { alDeleteSources(1, &source); }

    // Loads a buffer with an OGG. Returns 0 if loading was successful
    int LoadBuffer(ALuint buffer, std::string ogg_file);
    
    // Returns true if a source is playing. Else returns false
    bool SourceIsPlaying(ALuint source);
    
    void SetSourcePosition(ALuint source, vector3 pos);
    void SetSourceVelocity(ALuint source, vector3 vel);


private:

    ALCcontext* context_id;
    ALCdevice* dev;

};

#endif // defined(USE_SOUND)
#endif // AUDIOSYSTEMOPENAL_H
