// Slickworm
// Copyright (c) 2002-2004, Michel O'Brien.
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  1. Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//  2. Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//  3. Neither the names of the copyright holders nor the names of its
//     contributors may be used to endorse or promote products derived from this
//     software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
// IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#include "camera.h"
#include "gettime.h"

Camera::Camera(int time_lagged)
{
    this->time_lagged = time_lagged;
}

void Camera::setCurrentPosition(vector3 current_position)
{
    // Add the details of this camera position and the current time.
    camera_details cam;
    cam.pos = current_position;
    cam.time = GetTime();
    camera_positions.push_back(cam);
}

vector3 Camera::getSmooothedPosition()
{
    // Remove all the camera positions older than GetTime() - time_lagged.
    long threshold = GetTime() - time_lagged;

    std::deque<camera_details>::iterator i;

    // Probably better to use a std::for_each with some kind of predicate here.    
    for (i = camera_positions.begin(); i != camera_positions.end(); i++)
    {
        if (i->time < threshold)
        {
            camera_positions.erase(i);
            break;
        }
    }

    vector3 total(0.0, 0.0, 0.0);

    for (i = camera_positions.begin(); i != camera_positions.end(); i++)
    {
        total += i->pos;
    }

    total /= float(camera_positions.size());

    return total;
}

// Clear all the positions data
void Camera::clear()
{
    camera_positions.clear();
}
