// Slickworm
// Copyright (c) 2002-2004, Michel O'Brien.
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  1. Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//  2. Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//  3. Neither the names of the copyright holders nor the names of its
//     contributors may be used to endorse or promote products derived from this
//     software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
// IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#ifndef CAMERA_H
#define CAMERA_H

/*!
    \file camera.h
    \brief File containing Camera class.
*/

#include <deque>
#include "mtxlib.h"

// This camera system averages all the positions of the camera over a period
// of time to produce a smoother camera movement. Paul Nettle explained it
// better in one of his answers on Ask Midnight on Flipcode.

class Camera
{
public:
    Camera(int time_lagged);

    // Sets the current position of the camera
    void setCurrentPosition(vector3 current_position);

    // Gets the smoothed position
    vector3 getSmooothedPosition();

    // Clear all the positions data
    void clear();

private:
    // Time to hold old camera poitions
    int time_lagged;

    // A structure descibing the camera's position at a certain time.
    struct camera_details
    {
        vector3 pos;
        long time;
    };

    std::deque <camera_details> camera_positions;
};

#endif // CAMERA_H
