// Slickworm
// Copyright (c) 2002-2003, Michel O'Brien.
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  1. Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//  2. Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//  3. Neither the names of the copyright holders nor the names of its
//     contributors may be used to endorse or promote products derived from this
//     software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
// IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#ifndef EDITOR_MAIN_H
#define EDITOR_MAIN_H

#include "graphicssystem.h"
#include "level.h"
#include "editor_world.h"

/*!
    \file editor_main.h
    \brief File containing Level Editor class.
*/

class Editor
{
public:

    //! Constructor
    Editor(IGraphicsSystem* IGS, Level* level);

    //! Destructor
    ~Editor();

    //! Main update function. Call this once per frame.
    void Render(matrix44 perspective_matrix, matrix44 view_matrix);

    void RenderObject() { editor_world->RenderObject(); }

    vector3 GetCamera() { return camera; }
    vector3 GetLookat() { return lookat; }
    vector3 GetUp() { return up; }

private:

    EditorWorld* editor_world;

    vector3 camera, lookat, up;
    vector3 direction;
    float angle_y, angle_x, zoom;

    float horizontal_plane;
/*
    void DrawTransparentHorizontalPlane(float y);
*/
    IGraphicsSystem* IGS;

    TEXTURE_HANDLE white_texture;
};

#endif // EDITOR_MAIN_H

