// Slickworm
// Copyright (c) 2002-2004, Michel O'Brien.
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  1. Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//  2. Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//  3. Neither the names of the copyright holders nor the names of its
//     contributors may be used to endorse or promote products derived from this
//     software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
// IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#include "frustum.h"

// Code taken from Mark Morley's Frustum Culling tutorial.
// Code (C) 2000 Mark Morley.
// Cube test changed to AABB test by MOB.

// The 6 planes that describe the viewing frustum
float frustum[6][4];

void ExtractFrustum(matrix44 view, matrix44 prj)
{
    float proj[16];
    float modl[16];
    float clip[16];
    float t;

    // Convert prj to GL style matrix
    proj[0] = prj.col[0].x;
    proj[1] = prj.col[0].y;
    proj[2] = prj.col[0].z;
    proj[3] = prj.col[0].w;

    proj[4] = prj.col[1].x;
    proj[5] = prj.col[1].y;
    proj[6] = prj.col[1].z;
    proj[7] = prj.col[1].w;

    proj[8] = prj.col[2].x;
    proj[9] = prj.col[2].y;
    proj[10]= prj.col[2].z;
    proj[11]= prj.col[2].w;

    proj[12] = prj.col[3].x;
    proj[13] = prj.col[3].y;
    proj[14] = prj.col[3].z;
    proj[15] = prj.col[3].w;

    // Convert view to GL style matrix
    modl[0] = view.col[0].x;
    modl[1] = view.col[0].y;
    modl[2] = view.col[0].z;
    modl[3] = view.col[0].w;

    modl[4] = view.col[1].x;
    modl[5] = view.col[1].y;
    modl[6] = view.col[1].z;
    modl[7] = view.col[1].w;

    modl[8] = view.col[2].x;
    modl[9] = view.col[2].y;
    modl[10]= view.col[2].z;
    modl[11]= view.col[2].w;

    modl[12] = view.col[3].x;
    modl[13] = view.col[3].y;
    modl[14] = view.col[3].z;
    modl[15] = view.col[3].w;

    // Combine the two matrices (multiply projection by view_trasform)
    clip[ 0] = modl[ 0] * proj[ 0] + modl[ 1] * proj[ 4] + modl[ 2] * proj[ 8] + modl[ 3] * proj[12];
    clip[ 1] = modl[ 0] * proj[ 1] + modl[ 1] * proj[ 5] + modl[ 2] * proj[ 9] + modl[ 3] * proj[13];
    clip[ 2] = modl[ 0] * proj[ 2] + modl[ 1] * proj[ 6] + modl[ 2] * proj[10] + modl[ 3] * proj[14];
    clip[ 3] = modl[ 0] * proj[ 3] + modl[ 1] * proj[ 7] + modl[ 2] * proj[11] + modl[ 3] * proj[15];

    clip[ 4] = modl[ 4] * proj[ 0] + modl[ 5] * proj[ 4] + modl[ 6] * proj[ 8] + modl[ 7] * proj[12];
    clip[ 5] = modl[ 4] * proj[ 1] + modl[ 5] * proj[ 5] + modl[ 6] * proj[ 9] + modl[ 7] * proj[13];
    clip[ 6] = modl[ 4] * proj[ 2] + modl[ 5] * proj[ 6] + modl[ 6] * proj[10] + modl[ 7] * proj[14];
    clip[ 7] = modl[ 4] * proj[ 3] + modl[ 5] * proj[ 7] + modl[ 6] * proj[11] + modl[ 7] * proj[15];

    clip[ 8] = modl[ 8] * proj[ 0] + modl[ 9] * proj[ 4] + modl[10] * proj[ 8] + modl[11] * proj[12];
    clip[ 9] = modl[ 8] * proj[ 1] + modl[ 9] * proj[ 5] + modl[10] * proj[ 9] + modl[11] * proj[13];
    clip[10] = modl[ 8] * proj[ 2] + modl[ 9] * proj[ 6] + modl[10] * proj[10] + modl[11] * proj[14];
    clip[11] = modl[ 8] * proj[ 3] + modl[ 9] * proj[ 7] + modl[10] * proj[11] + modl[11] * proj[15];

    clip[12] = modl[12] * proj[ 0] + modl[13] * proj[ 4] + modl[14] * proj[ 8] + modl[15] * proj[12];
    clip[13] = modl[12] * proj[ 1] + modl[13] * proj[ 5] + modl[14] * proj[ 9] + modl[15] * proj[13];
    clip[14] = modl[12] * proj[ 2] + modl[13] * proj[ 6] + modl[14] * proj[10] + modl[15] * proj[14];
    clip[15] = modl[12] * proj[ 3] + modl[13] * proj[ 7] + modl[14] * proj[11] + modl[15] * proj[15];

    // Extract the numbers for the RIGHT plane
    frustum[0][0] = clip[ 3] - clip[ 0];
    frustum[0][1] = clip[ 7] - clip[ 4];
    frustum[0][2] = clip[11] - clip[ 8];
    frustum[0][3] = clip[15] - clip[12];

    // Normalize the result
    t = sqrt( frustum[0][0] * frustum[0][0] + frustum[0][1] * frustum[0][1] + frustum[0][2] * frustum[0][2] );
    frustum[0][0] /= t;
    frustum[0][1] /= t;
    frustum[0][2] /= t;
    frustum[0][3] /= t;

    // Extract the numbers for the LEFT plane
    frustum[1][0] = clip[ 3] + clip[ 0];
    frustum[1][1] = clip[ 7] + clip[ 4];
    frustum[1][2] = clip[11] + clip[ 8];
    frustum[1][3] = clip[15] + clip[12];

    // Normalize the result
    t = sqrt( frustum[1][0] * frustum[1][0] + frustum[1][1] * frustum[1][1] + frustum[1][2] * frustum[1][2] );
    frustum[1][0] /= t;
    frustum[1][1] /= t;
    frustum[1][2] /= t;
    frustum[1][3] /= t;

    // Extract the BOTTOM plane
    frustum[2][0] = clip[ 3] + clip[ 1];
    frustum[2][1] = clip[ 7] + clip[ 5];
    frustum[2][2] = clip[11] + clip[ 9];
    frustum[2][3] = clip[15] + clip[13];

    // Normalize the result
    t = sqrt( frustum[2][0] * frustum[2][0] + frustum[2][1] * frustum[2][1] + frustum[2][2] * frustum[2][2] );
    frustum[2][0] /= t;
    frustum[2][1] /= t;
    frustum[2][2] /= t;
    frustum[2][3] /= t;

    // Extract the TOP plane
    frustum[3][0] = clip[ 3] - clip[ 1];
    frustum[3][1] = clip[ 7] - clip[ 5];
    frustum[3][2] = clip[11] - clip[ 9];
    frustum[3][3] = clip[15] - clip[13];

    // Normalize the result
    t = sqrt( frustum[3][0] * frustum[3][0] + frustum[3][1] * frustum[3][1] + frustum[3][2] * frustum[3][2] );
    frustum[3][0] /= t;
    frustum[3][1] /= t;
    frustum[3][2] /= t;
    frustum[3][3] /= t;

    // Extract the FAR plane
    frustum[4][0] = clip[ 3] - clip[ 2];
    frustum[4][1] = clip[ 7] - clip[ 6];
    frustum[4][2] = clip[11] - clip[10];
    frustum[4][3] = clip[15] - clip[14];

    // Normalize the result
    t = sqrt( frustum[4][0] * frustum[4][0] + frustum[4][1] * frustum[4][1] + frustum[4][2] * frustum[4][2] );
    frustum[4][0] /= t;
    frustum[4][1] /= t;
    frustum[4][2] /= t;
    frustum[4][3] /= t;

    // Extract the NEAR plane
    frustum[5][0] = clip[ 3] + clip[ 2];
    frustum[5][1] = clip[ 7] + clip[ 6];
    frustum[5][2] = clip[11] + clip[10];
    frustum[5][3] = clip[15] + clip[14];

    // Normalize the result
    t = sqrt( frustum[5][0] * frustum[5][0] + frustum[5][1] * frustum[5][1] + frustum[5][2] * frustum[5][2] );
    frustum[5][0] /= t;
    frustum[5][1] /= t;
    frustum[5][2] /= t;
    frustum[5][3] /= t;
}

bool PointInFrustum(vector3 point)
{
    int p;

    for( p = 0; p < 6; p++ )
    {
        if( frustum[p][0] * point.x + frustum[p][1] * point.y + frustum[p][2] * point.z + frustum[p][3] <= 0 )
        {
            return false;
        }
    }
    return true;
}

bool SphereInFrustum(vector3 centre, float radius )
{
    int p;

    for( p = 0; p < 6; p++ )
    {
        if( frustum[p][0] * centre.x + frustum[p][1] * centre.y + frustum[p][2] * centre.z + frustum[p][3] <= -radius )
        {
            return false;
        }
    }
    return true;
}

bool AABBInFrustum(float aabb[6])
{
    int p;

    for( p = 0; p < 6; p++ )
    {
        if( frustum[p][0] * aabb[0] + frustum[p][1] * aabb[2] + frustum[p][2] * aabb[4] + frustum[p][3] > 0 )
            continue;
        if( frustum[p][0] * aabb[1] + frustum[p][1] * aabb[2] + frustum[p][2] * aabb[4] + frustum[p][3] > 0 )
            continue;
        if( frustum[p][0] * aabb[0] + frustum[p][1] * aabb[3] + frustum[p][2] * aabb[4] + frustum[p][3] > 0 )
            continue;
        if( frustum[p][0] * aabb[1] + frustum[p][1] * aabb[3] + frustum[p][2] * aabb[4] + frustum[p][3] > 0 )
            continue;
        if( frustum[p][0] * aabb[0] + frustum[p][1] * aabb[2] + frustum[p][2] * aabb[5] + frustum[p][3] > 0 )
            continue;
        if( frustum[p][0] * aabb[1] + frustum[p][1] * aabb[2] + frustum[p][2] * aabb[5] + frustum[p][3] > 0 )
            continue;
        if( frustum[p][0] * aabb[0] + frustum[p][1] * aabb[3] + frustum[p][2] * aabb[5] + frustum[p][3] > 0 )
            continue;
        if( frustum[p][0] * aabb[1] + frustum[p][1] * aabb[3] + frustum[p][2] * aabb[5] + frustum[p][3] > 0 )
            continue;
        return false;
    }
    return true;
}


