// Slickworm
// Copyright (c) 2002-2004, Michel O'Brien.
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  1. Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//  2. Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//  3. Neither the names of the copyright holders nor the names of its
//     contributors may be used to endorse or promote products derived from this
//     software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
// IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#include "keyboard.h"

Keyboard::Keyboard()
{
    // Start off as if all keys are up.
    for (int k = 0; k < NUMBER_OF_KEYS; k++)
    {
        key_state[k] = false;
        switch_state[k] = false;
    }
}

// Returns true if the key corresponding to key is down. Returns false otherwise.
bool Keyboard::KeyPressed(SDLKey key)
{
    return key_state[key];
}

// Queries the current value of a switch corresponding to a key
bool Keyboard::QueryCurrentValueOfSwitch(SDLKey key)
{
    return switch_state[key];
}

// To be called whenever an SDL_KEYUP or SDL_KEYDOWN event is received by the main loop.
void Keyboard::KeyEventNotify(SDL_KeyboardEvent *keyevent)
{
    if (keyevent->type == SDL_KEYDOWN)
    {
        key_state[keyevent->keysym.sym] = true;

        // Toggle the switch state
        if (switch_state[keyevent->keysym.sym] == false)
        {
            switch_state[keyevent->keysym.sym] = true;
        }
        else // switch_state[keyevent->keysym.sym] == true
        {
            switch_state[keyevent->keysym.sym] = false;
        }
    }
    else if (keyevent->type == SDL_KEYUP)
    {
        key_state[keyevent->keysym.sym] = false;
    }
}




