// Slickworm
// Copyright (c) 2002-2004, Michel O'Brien.
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  1. Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//  2. Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//  3. Neither the names of the copyright holders nor the names of its
//     contributors may be used to endorse or promote products derived from this
//     software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
// IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#ifndef PARTICLE_H
#define PARTICLE_H

#include "ode_thread.h"
#include "mtxlib.h"
#include "model.h"
#include <string>

enum object_type
{
    SPHERE,
    BOX,
    CAPSULE
};

class Particle
{
public:
    //! Constructor
    Particle(IGraphicsSystem* IGS, dWorldID world, dSpaceID space, vector3 position, vector3 velocity,
        std::string model_directory, std::string model_filename, IModel* model, object_type type, vector3 size);

    //! Destructor
    ~Particle();

    void Render();

    //! Returns the current position of the particle.
    vector3 GetPosition();

    //! Returns the spawn time of this particle.
    long GetSpawnTime() { return spawn_time; }

    dBodyID GetBody() { return body; }
    dGeomID GetGeom() { return geom; }

private:

    long spawn_time;

    IGraphicsSystem* IGS;

    dWorldID world;
    dSpaceID space;

    IModel* model;
    bool delete_model;

    dBodyID body;
    dGeomID geom;
};

#endif // PARTICLE_H



