// Slickworm
// Copyright (c) 2002-2004, Michel O'Brien.
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  1. Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//  2. Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//  3. Neither the names of the copyright holders nor the names of its
//     contributors may be used to endorse or promote products derived from this
//     software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
// IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#include "object_renderer.h"
#include "gettime.h"
#include "particle_system.h"

ParticleSystem::ParticleSystem(IGraphicsSystem* igs, vector3 position, vector3 velocity, Particle** emitter)
{
    this->emitter = emitter;
    this->IGS = igs;

    // Store the creation time of this particle system.
    this->spawn_time = GetTime();

    for (int p = 0; p < 100; p++)
    {
        particles[p] = NULL;
    }

    last_spawned_particle = 0;

    this->position = position;
    this->velocity = velocity;
}

//! Destructor
ParticleSystem::~ParticleSystem()
{
    for (int p = 0; p < 100; p++)
    {
        if (particles[p] != NULL)
        {
            delete particles[p];
        }
    }
}

void ParticleSystem::Update()
{
    vector3 spawn_position;

    if (emitter != NULL)
    {
        if (*emitter != NULL)
        {
            spawn_position = (*emitter)->GetPosition();
            last_position = spawn_position;
        }
        else
        {
            spawn_position = last_position;

            last_spawned_particle = GetTime();
        }

        for (int p1 = 0; p1 < 100; p1++)
        {
            if (particles[p1] == NULL && (GetTime() - last_spawned_particle) > 40)
            {
                particles[p1] = new ManagedParticle;
                particles[p1]->spawn_time = GetTime();
                particles[p1]->position = spawn_position;

                last_spawned_particle = GetTime();
            }
        }

        for (int p2 = 0; p2 < 100; p2++)
        {
            if (particles[p2] != NULL)
            {
                if (GetTime() - particles[p2]->spawn_time > 1000)
                {
                    delete particles[p2];
                    particles[p2] = NULL;
                }
            }
        }
    }

    if (emitter == NULL)
    {
        if (last_spawned_particle == 0)
        {
            for (int p = 0; p < 10; p++)
            {
                particles[p] = new ManagedParticle;
                particles[p]->spawn_time = GetTime();
                particles[p]->position = position + velocity * ((float)p * 1.0f) + vector3(rand()%100 / 500.0f, rand()%100 / 500.0f, rand()%100 / 500.0f);

                last_spawned_particle = GetTime();
            }
        }
    }
}

void ParticleSystem::Render(TEXTURE_HANDLE smoke0_texture)
{
/*
    if (emitter != NULL)
    {
        for (int p = 0; p < 100; p++)
        {
            if (particles[p] != NULL)
            {
                float angle = (float)(GetTime() % (360 * 25)) / 25.0f;
                float alpha = (1000 - (GetTime() - particles[p]->spawn_time)) / 4000.0f;
                float size = (GetTime() - particles[p]->spawn_time) / 500.0f + 0.4f;
                vector3 colour = vector3(0.6f, 0.6f, 0.6f) + vector3(0.2f, 0.2f, 0.2f) * (1000 - (GetTime() - particles[p]->spawn_time)) / 1000.0f;
                ObjectRenderer::Instance().AddParticle(size, COLOUR(colour.x, colour.y, colour.z, alpha), SRC_ALPHA_ONE_MINUS_SRC_ALPHA, smoke0_texture, particles[p]->position, angle);
            }
        }
    }
*/
    if (emitter != NULL)
    {
        for (int p = 0; p < 100; p++)
        {
            if (particles[p] != NULL)
            {
                float angle = (float)(GetTime() % (360 * 25)) / 25.0f;
                float alpha = (1000 - (GetTime() - particles[p]->spawn_time)) / 3000.0f;
                float size = (GetTime() - particles[p]->spawn_time) / 200.0f + 0.6f;
                vector3 colour = vector3(0.2f, 0.1f, 1.0f) + vector3(0.5f, 0.0f, 0.0f) * (1000 - (GetTime() - particles[p]->spawn_time)) / 1000.0f;
                ObjectRenderer::Instance().AddParticle(size, COLOUR(colour.x, colour.y, colour.z, alpha), SRC_ALPHA_ONE, smoke0_texture, particles[p]->position, angle);
            }
        }
    }





    if (emitter == NULL)
    {
        if (GetTime() - spawn_time < 100)
        {
            float angle = (float)(GetTime() % (360 * 25)) / 25.0f;
            float alpha = (100 - (GetTime() - spawn_time)) / 100.0f;
            ObjectRenderer::Instance().AddParticle(0.3f, COLOUR(1.0f, 1.0f, 1.0f, alpha), SRC_ALPHA_ONE, smoke0_texture, this->position, angle);
            angle = (float)(GetTime() % (360 * 25)) / 25.0f;
            alpha = (100 - (GetTime() - spawn_time)) / 100.0f;
            ObjectRenderer::Instance().AddParticle(0.7f, COLOUR(1.0f, 0.4f, 0.1f, alpha), SRC_ALPHA_ONE, smoke0_texture, this->position + this->velocity * 0.3f, angle);
        }

        if (GetTime() - spawn_time < 1000)
        {
            for (int p = 0; p < 10; p++)
            {
                if (particles[p] != NULL)
                {
                    float angle = (float)(GetTime() % (360 * 25)) / 25.0f;
                    float alpha = (1000 - (GetTime() - particles[p]->spawn_time)) / 1000.0f;
                    float size = (GetTime() - particles[p]->spawn_time) / 1500.0f;
                    ObjectRenderer::Instance().AddParticle(size, COLOUR(0.8f, 0.8f, 0.8f, alpha), SRC_ALPHA_ONE_MINUS_SRC_ALPHA, smoke0_texture, particles[p]->position, angle);
                }
            }
        }
    }

}






