// Slickworm
// Copyright (c) 2002-2003, Michel O'Brien.
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  1. Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//  2. Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//  3. Neither the names of the copyright holders nor the names of its
//     contributors may be used to endorse or promote products derived from this
//     software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
// IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#ifndef WORLD_H
#define WORLD_H

#include <string>
#include "mtxlib.h"
#include "model.h"

class World
{
public:
    //! Constructor
    World(IGraphicsSystem* IGS);

    ~World();

    void Render();

    void New();

    void Load(std::string filename);

    void Save(std::string filename);

    void AddInstance(std::string filename, vector3 v = vector3(0.0, 0.0, 0.0), float rotation_y = 0.0);

private:

    IGraphicsSystem* IGS;

    dWorldID world;
    dSpaceID space;

    class Location
    {
    public:
        Location(IModel* model, vector3 v = vector3(0.0, 0.0, 0.0), float rotation_y = 0.0);
        vector3 position;
        float rotation_y;
        dGeomID model_geom;
    };

    class Object
    {
    public:
        Object(std::string filename, IGraphicsSystem* IGS);
        std::string model_filename;
        IModel* model;
        std::vector<Location> instances;
    };

    std::vector<Object> objects;





};


#endif // WORLD_H

